@@ -2106,9 +2106,13 @@ class NiceTracer(Tracer):
             add_tag(root.body[1], "ignore")
 
         for node in ast.walk(root):
-
+            if not isinstance(node, (ast.expr, ast.stmt)):
+                continue
+            
             # tag last children
             last_child = ast_utils.get_last_child(node)
+            assert last_child in [True, False, None] or isinstance(last_child, (ast.expr, ast.stmt, type(None))), \
+                "Bad last child " + str(last_child) + " of " + str(node)
             if last_child is not None:
                 add_tag(node, "has_children")
 
@@ -2435,6 +2439,7 @@ class NiceTracer(Tracer):
         )
 
     def _export_node(self, node):
+        assert isinstance(node, (ast.expr, ast.stmt))
         node_id = id(node)
         self._nodes[node_id] = node
         return ast.Num(node_id)
