@@ -31,7 +31,8 @@ from thonny import (
     get_workbench,
     ui_utils,
 )
-from thonny.code import get_current_breakpoints
+from thonny.code import get_current_breakpoints,\
+    get_saved_current_script_filename
 from thonny.common import (
     BackendEvent,
     CommandToBackend,
@@ -47,10 +48,12 @@ from thonny.common import (
     parse_message,
     path_startswith,
     serialize_message,
-)
+    update_system_path)
 from thonny.misc_utils import construct_cmd_line, running_on_mac_os, running_on_windows
 
 from typing import Any, List, Optional, Sequence, Set  # @UnusedImport; @UnusedImport
+from thonny.terminal import run_in_terminal
+from thonny.ui_utils import select_sequence
 
 
 WINDOWS_EXE = "python.exe"
@@ -81,6 +84,9 @@ class Runner:
         self._remove_obsolete_jedi_copies()
 
     def _init_commands(self) -> None:
+        get_workbench().set_default("run.run_in_terminal_python_repl", False)
+        get_workbench().set_default("run.run_in_terminal_keep_open", True)
+        
         get_workbench().add_command(
             "run_current_script",
             "run",
@@ -88,10 +94,25 @@ class Runner:
             caption="Run",
             handler=self._cmd_run_current_script,
             default_sequence="<F5>",
+            extra_sequences=[select_sequence("<Control-r>", "<Command-r>")],
             tester=self._cmd_run_current_script_enabled,
             group=10,
             image="run-current-script",
             include_in_toolbar=True,
+            show_extra_sequences=True,
+        )
+
+        get_workbench().add_command(
+            "run_current_script_in_terminal",
+            "run",
+            "Run current script in terminal",
+            caption="RunT",
+            handler=self._cmd_run_current_script_in_terminal,
+            default_sequence="<Control-t>",
+            extra_sequences=["<<CtrlTInText>>"],
+            tester=self._cmd_run_current_script_in_terminal_enabled,
+            group=35,
+            image="terminal",
         )
 
         get_workbench().add_command(
@@ -254,19 +275,8 @@ class Runner:
 
         if not self.is_waiting_toplevel_command():
             self.restart_backend(False, False, 2)
-
-        editor = get_workbench().get_editor_notebook().get_current_editor()
-        if not editor:
-            return
-
-        filename = editor.get_filename(True)
-        if not filename:
-            return
-
-        if editor.is_modified():
-            filename = editor.save_file()
-            if not filename:
-                return
+        
+        filename = get_saved_current_script_filename()
 
         # changing dir may be required
         script_dir = normpath_with_actual_case(os.path.dirname(filename))
@@ -279,25 +289,42 @@ class Runner:
             working_directory = script_dir  # type: Optional[str]
         else:
             working_directory = None
-
-        if get_workbench().get_option("view.show_program_arguments"):
-            args_str = get_workbench().get_option("run.program_arguments")
-            get_workbench().log_program_arguments_string(args_str)
-            args = shlex.split(args_str)
-        else:
-            args = []
+        
+        args = self._get_active_arguments()
 
         self.execute_script(filename, args, working_directory, command_name)
 
+    def _get_active_arguments(self):
+        if get_workbench().get_option("view.show_program_arguments"):
+            args_str = get_workbench().get_option("run.program_arguments")
+            get_workbench().log_program_arguments_string(args_str)
+            return shlex.split(args_str)
+        else:
+            return []
+    
     def _cmd_run_current_script_enabled(self) -> bool:
         return (
             get_workbench().get_editor_notebook().get_current_editor() is not None
-            and "run" in get_runner().supported_features()
+            and "run" in get_runner().get_supported_features()
         )
 
+    def _cmd_run_current_script_in_terminal_enabled(self) -> bool:
+        return (self._proxy 
+                and "run_in_terminal" in self._proxy.get_supported_features()
+                and self._cmd_run_current_script_enabled())
+
     def _cmd_run_current_script(self) -> None:
         self.execute_current("Run")
 
+    def _cmd_run_current_script_in_terminal(self) -> None:
+        filename = get_saved_current_script_filename()
+        self._proxy.run_script_in_terminal(
+            filename,
+            self._get_active_arguments(),
+            get_workbench().get_option("run.run_in_terminal_python_repl"),
+            get_workbench().get_option("run.run_in_terminal_keep_open"),
+        )
+
     def _cmd_interrupt(self) -> None:
         if self._proxy is not None:
             self._proxy.interrupt()
@@ -454,11 +481,11 @@ class Runner:
             self._proxy.destroy()
             self._proxy = None
 
-    def get_executable(self) -> Optional[str]:
+    def get_local_executable(self) -> Optional[str]:
         if self._proxy is None:
             return None
         else:
-            return self._proxy.get_executable()
+            return self._proxy.get_local_executable()
 
     def get_backend_proxy(self) -> "BackendProxy":
         return self._proxy
@@ -504,11 +531,11 @@ class Runner:
                     "Problem with finalizing console allocation"
                 )
 
-    def supported_features(self) -> Set[str]:
+    def get_supported_features(self) -> Set[str]:
         if self._proxy is None:
             return set()
         else:
-            return self._proxy.supported_features()
+            return self._proxy.get_supported_features()
 
     def using_venv(self) -> bool:
         return isinstance(self._proxy, CPythonProxy) and self._proxy.in_venv
@@ -547,6 +574,9 @@ class BackendProxy:
         """Read next message from the queue or None if queue is empty"""
         raise NotImplementedError()
 
+    def run_script_in_terminal(self, script_path, interactive, keep_open):
+        raise NotImplementedError()
+    
     def get_sys_path(self):
         "backend's sys.path"
         return []
@@ -568,12 +598,14 @@ class BackendProxy:
         """Used in MicroPython proxies"""
         return True
 
-    def get_executable(self):
+    def get_local_executable(self):
         """Return system command for invoking current interpreter"""
         return None
 
-    def supported_features(self):
+    def get_supported_features(self):
         return {"run"}
+    
+    
 
 
 class CPythonProxy(BackendProxy):
@@ -600,20 +632,7 @@ class CPythonProxy(BackendProxy):
             return None
 
         msg = self._message_queue.popleft()
-        if "gui_is_active" in msg:
-            self._update_gui_updating(msg)
-
-        if "in_venv" in msg:
-            self.in_venv = msg["in_venv"]
-
-        if "path" in msg:
-            self._sys_path = msg["path"]
-
-        if "usersitepackages" in msg:
-            self._usersitepackages = msg["usersitepackages"]
-
-        if "prefix" in msg:
-            self._sys_prefix = msg["prefix"]
+        self._store_state_info(msg)
 
         if msg.event_type == "ProgramOutput":
             # combine available output messages to one single message,
@@ -636,6 +655,25 @@ class CPythonProxy(BackendProxy):
 
         else:
             return msg
+    
+    def _store_state_info(self, msg):
+        if "gui_is_active" in msg:
+            self._update_gui_updating(msg)
+
+        if "in_venv" in msg:
+            self.in_venv = msg["in_venv"]
+
+        if "path" in msg:
+            self._sys_path = msg["path"]
+
+        if "usersitepackages" in msg:
+            self._usersitepackages = msg["usersitepackages"]
+
+        if "prefix" in msg:
+            self._sys_prefix = msg["prefix"]
+        
+        if "exe_dirs" in msg:
+            self._exe_dirs = msg["exe_dirs"]
 
     def send_command(self, cmd):
         if isinstance(cmd, ToplevelCommand) and cmd.name[0].isupper():
@@ -679,36 +717,11 @@ class CPythonProxy(BackendProxy):
         self._message_queue = None
 
     def _start_new_process(self, cmd=None):
-        this_python = get_frontend_python()
         # deque, because in one occasion I need to put messages back
         self._message_queue = collections.deque()
 
-        # prepare the environment
-        my_env = os.environ.copy()
-
-        # Delete some environment variables if the backend is (based on) a different Python instance
-        if self._executable not in [
-            this_python,
-            this_python.replace("python.exe", "pythonw.exe"),
-            this_python.replace("pythonw.exe", "python.exe"),
-        ]:
-
-            # Keep only the variables, that are not related to Python
-            my_env = {
-                name: my_env[name]
-                for name in my_env
-                if "python" not in name.lower()
-                and name not in ["TK_LIBRARY", "TCL_LIBRARY"]
-            }
-
-            # Remove variables used to tweak bundled Thonny-private Python
-            if using_bundled_python():
-                my_env = {
-                    name: my_env[name]
-                    for name in my_env
-                    if name not in ["SSL_CERT_FILE", "SSL_CERT_DIR", "LD_LIBRARY_PATH"]
-                }
-
+        # prepare environment
+        my_env = get_environment_for_python_subprocess(self._executable)
         # variables controlling communication with the back-end process
         my_env["PYTHONIOENCODING"] = "utf-8"
 
@@ -720,14 +733,6 @@ class CPythonProxy(BackendProxy):
         elif "THONNY_DEBUG" in my_env:
             del my_env["THONNY_DEBUG"]
 
-        # venv may not find (correct) Tk without assistance (eg. in Ubuntu)
-        if self._executable == get_private_venv_executable():
-            try:
-                my_env["TCL_LIBRARY"] = get_workbench().tk.exprstring("$tcl_library")
-                my_env["TK_LIBRARY"] = get_workbench().tk.exprstring("$tk_library")
-            except Exception:
-                logging.exception("Can't find Tcl/Tk library")
-
         if not os.path.exists(self._executable):
             raise UserError(
                 "Interpreter (%s) not found. Please recheck corresponding option!"
@@ -778,6 +783,7 @@ class CPythonProxy(BackendProxy):
             if ready_line == "":  # There was some problem
                 error_msg = self._proc.stderr.read()
                 raise Exception("Error starting backend process: " + error_msg)
+            self._store_state_info(parse_message(ready_line))
 
         # setup asynchronous output listeners
         Thread(target=self._listen_stdout, daemon=True).start()
@@ -846,7 +852,7 @@ class CPythonProxy(BackendProxy):
                     BackendEvent("ProgramOutput", stream_name="stderr", data=data)
                 )
 
-    def get_executable(self):
+    def get_local_executable(self):
         return self._executable
 
     def get_site_packages(self):
@@ -861,6 +867,9 @@ class CPythonProxy(BackendProxy):
 
     def get_user_site_packages(self):
         return self._usersitepackages
+    
+    def get_exe_dirs(self):
+        return self._exe_dirs
 
     def _update_gui_updating(self, msg):
         """Enables running Tkinter or Qt programs which doesn't call mainloop. 
@@ -893,8 +902,18 @@ class CPythonProxy(BackendProxy):
             finally:
                 self._gui_update_loop_id = None
 
-    def supported_features(self):
-        return {"run", "debug", "pip_gui", "system_shell"}
+    def run_script_in_terminal(self, script_path, args, interactive, keep_open):
+        cmd = [self._executable]
+        if interactive:
+            cmd.append("-i")
+        cmd.append(os.path.basename(script_path))
+        cmd.extend(args)
+        
+        run_in_terminal(cmd, os.path.dirname(script_path),
+                        keep_open=keep_open)
+    
+    def get_supported_features(self):
+        return {"run", "debug", "run_in_terminal", "pip_gui", "system_shell"}
 
 
 class PrivateVenvCPythonProxy(CPythonProxy):
@@ -1058,23 +1077,14 @@ def _get_venv_info(venv_path):
 
 
 def using_bundled_python():
+    return is_bundled_python(sys.executable)
+
+def is_bundled_python(executable):
     return os.path.exists(
-        os.path.join(os.path.dirname(sys.executable), "thonny_python.ini")
+        os.path.join(os.path.dirname(executable), "thonny_python.ini")
     )
 
 
-def create_pythonless_environment():
-    # If I want to call another python version, then
-    # I need to remove from environment the items installed by current interpreter
-    env = {}
-
-    for key in os.environ:
-        if "python" not in key.lower() and key not in ["TK_LIBRARY", "TCL_LIBRARY"]:
-            env[key] = os.environ[key]
-
-    return env
-
-
 def create_backend_python_process(
     args, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
 ):
@@ -1083,9 +1093,9 @@ def create_backend_python_process(
 
     # TODO: if backend == frontend, then delegate to create_frontend_python_process
 
-    python_exe = get_runner().get_executable()
+    python_exe = get_runner().get_local_executable()
 
-    env = create_pythonless_environment()
+    env = get_environment_for_python_subprocess(python_exe)
     env["PYTHONIOENCODING"] = "utf-8"
     env["PYTHONUNBUFFERED"] = "1"
 
@@ -1099,6 +1109,9 @@ def create_frontend_python_process(
 ):
     """Used for running helper commands (eg. for installing plug-ins on by the plug-ins)"""
     python_exe = get_frontend_python().replace("pythonw.exe", "python.exe")
+    env = get_environment_for_python_subprocess(python_exe)
+    env["PYTHONIOENCODING"] = "utf-8"
+    env["PYTHONUNBUFFERED"] = "1"
     return _create_python_process(python_exe, args, stdin, stdout, stderr)
 
 
@@ -1147,3 +1160,99 @@ class BackendTerminatedError(Exception):
 
 def get_frontend_python():
     return sys.executable.replace("thonny.exe", "python.exe")
+
+def is_venv_interpreter_of_current_interpreter(executable):
+    for location in [".", ".."]:
+        cfg_path = os.path.join(location, "pyvenv.cfg")
+        if os.path.isfile(cfg_path):
+            with open(cfg_path) as fp:
+                content = fp.read()
+            for line in content.splitlines():
+                if line.replace(" ", "").startswith("home="):
+                    _, home = line.split("=", maxsplit=1)
+                    home = home.strip()
+                    if os.path.isdir(home) and os.path.samefile(home, sys.prefix):
+                        return True
+    return False
+
+    
+def get_environment_for_python_subprocess(target_executable):
+    overrides = get_environment_overrides_for_python_subprocess(target_executable)
+    return get_environment_with_overrides(overrides)
+
+def get_environment_with_overrides(overrides):
+    env = os.environ.copy()
+    for key in overrides:
+        if overrides[key] is None and key in env:
+            del env[key]
+        else:
+            assert isinstance(overrides[key], str)
+            if key.upper() == "PATH":
+                update_system_path(env, overrides[key])
+            else:
+                env[key] = overrides[key]
+    return env
+    
+def get_environment_overrides_for_python_subprocess(target_executable):
+    """Take care of not not confusing different interpreter 
+    with variables meant for bundled interpreter"""
+    
+    # At the moment I'm tweaking the environment only if current 
+    # exe is bundled for Thonny. 
+    # In remaining cases it is user's responsibility to avoid 
+    # calling Thonny with environment which may be confusing for 
+    # different Pythons called in a subprocess.
+    
+    this_executable = sys.executable.replace("pythonw.exe", "python.exe")
+    target_executable = target_executable.replace("pythonw.exe", "python.exe")
+    
+    interpreter_specific_keys = ["TCL_LIBRARY", "TK_LIBRARY",
+                                 "LD_LIBRARY_PATH", "DYLD_LIBRARY_PATH",
+                                 "SSL_CERT_DIR", "SSL_CERT_FILE",
+                                 "PYTHONHOME", "PYTHONPATH",
+                                 "PYTHONNOUSERSITE", "PYTHONUSERBASE"]
+    
+    result = {}
+    
+    if (os.path.samefile(target_executable, this_executable)
+        or is_venv_interpreter_of_current_interpreter(target_executable)):
+        # bring out some important variables so that they can 
+        # be explicitly set in macOS Terminal
+        # (If they are set then it's most likely because current exe is in Thonny bundle)
+        for key in interpreter_specific_keys:
+            if key in os.environ:
+                result[key] = os.environ[key]
+        
+        # never pass some variables to different interpreter
+        # (even if it's venv or symlink to current one)
+        if not is_same_path(target_executable, this_executable):
+            for key in ["PYTHONPATH", "PYTHONHOME",
+                        "PYTHONNOUSERSITE", "PYTHONUSERBASE"]:
+                if key in os.environ:
+                    result[key] = None
+    else:
+        # interpreters are not related
+        # interpreter specific keys most likely would confuse other interpreter
+        for key in interpreter_specific_keys:
+            if key in os.environ:
+                result[key] = None
+    
+    
+    # some keys should be never passed
+    for key in ["PYTHONSTARTUP", "PYTHONBREAKPOINT", "PYTHONDEBUG",
+                "PYTHONNOUSERSITE", "PYTHONASYNCIODEBUG"]:
+        if key in os.environ:
+            result[key] = None
+
+    # venv may not find (correct) Tk without assistance (eg. in Ubuntu)
+    if is_venv_interpreter_of_current_interpreter(target_executable):
+        try:
+            if ("TCL_LIBRARY" not in os.environ
+                or "TK_LIBRARY" not in os.environ): 
+                result["TCL_LIBRARY"] = get_workbench().tk.exprstring("$tcl_library")
+                result["TK_LIBRARY"] = get_workbench().tk.exprstring("$tk_library")
+        except Exception:
+            logging.exception("Can't compute Tcl/Tk library location")
+
+    
+    return result
