@@ -13,10 +13,11 @@ else:
 
 
 def test_regular_closed():
-    insert_pos_groups = (("1.9", "1.10", "1.13", "1.31"),
-                         ("1.30", "1.29", "1.25", "1.15"))
-    expected_indices = (("1.9", "1.30", []),
-                        ("1.15", "1.29", []))
+    insert_pos_groups = (
+        ("1.9", "1.10", "1.13", "1.31"),
+        ("1.30", "1.29", "1.25", "1.15"),
+    )
+    expected_indices = (("1.9", "1.30", []), ("1.15", "1.29", []))
 
     text_widget = tkinter.Text()
     text_widget.insert("end", TEST_STR1)
@@ -31,13 +32,4 @@ def test_regular_closed():
             expected = expected_indices[i]
 
             assert actual == expected, "\nExpected: %s\nGot: %s" % (expected, actual)
-        print("\rPassed %d of %d" % (i+1, len(insert_pos_groups)), end="")
-
-
-def run_tests():
-    test_regular_closed()
-
-if __name__ == "__main__":
-    print("Test input: ")
-    print(TEST_STR1)
-    run_tests()
+        print("\rPassed %d of %d" % (i + 1, len(insert_pos_groups)), end="")
